' WS2812 TRAFFICLIGHT DEMO
' by "Tubular", May 2014

' Demo of driving WS2811, -12, -12B/"13", "NeoPixels"(tm?) using 2 wire technique.
' +VE and DATA combined on single pin, allows 2 wires to remote pixel array (GND and combined +VE/DIN)
' Use with small number of pixels eg 3~12 "PolyPixels"

' At remote end, Use a Schottky rectifier diode from DIN to +5V.  And big cap (10uF?) from rectified "+5V" to GND to hold charge and reduce flicker

' Prop output resistance ~28 ohms

'P24 -----+--------------------------+---------- DIN   WS2812B strip end
'P25 -----|                          | Diode           use Schottky diode
'P26 -----|                          '--|>|--+-- "+5V" will rectify to ~2.5v DC if using 3v3 prop supply (depends how many pixels, how many driven on, etc)
'P27 -----' (parallel opt'l)                 | 
'                                       Cap ===        eg 10uF
'VSS ________________________________________|__ VSS   

' Trick: Uses several "phantom white pixels" after the real pixels, to boost the average voltage (more 1's in the data stream)
' Trick: Use the chassis (where available) for the VSS return, then only need to run one wire to each short strip.
' You can 'power down' (stop toggling) the led strip when not in use. They normally consume ~0.5mA per pixel at 2.5v.

' Driver based on JM_WS2812_demo by Jon McPhalen
' Several changes to jm_ws2812 driver (now called jm_ws2812b) - longer +ve high time.  Much shorter reset time. 4 pins driven in parallel.


con { timing }

  _clkmode = xtal1 + pll16x                                     ' 16x required for WS2812                              
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq               ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con { io pins }

  RX1  = 31                                                     ' programming / terminal
  TX1  = 30
  
  SDA  = 29                                                     ' eeprom / i2c
  SCL  = 28

  LEDS = 6                                                     ' LED tx pin.   NOTE NOW DRIVES 4 SUCCESSIVE PINS IN PARALLEL, IE P24..P27. SEE JM_WS2812
           
  RealPixels = 7                                               ' how many pixels are actually connected
  TotalPixels = 84                                             ' total number of pixels in data stream.  Allow extra for:-
  PhantomPixels = TotalPixels - RealPixels                     ' Phantom pixels are "full white" to increase the number of 1's in the stream, and hence avg voltage
  
obj
  strip : "jm_ws2812b"                                           ' WS2812 LED driver.  Modified timing and drives 4 parallel outputs

pub main | pos
  strip.start(LEDS, TotalPixels)                                 ' start led driver
  pause(10)
  repeat
    strip.set_rgb(0,0,30,30)    'green with a bit of blue just like real traffic lights do
    strip.set_rgb(5,0,30,30)    'green on second set on back of 1st set
    strip.set_rgb(2,0,0,0)      'turn off reds
    strip.set_rgb(3,0,0,0)
    repeat pos from RealPixels to TotalPixels
      strip.set_rgb(pos,255,255,255)                             ' phantom white pixels boost the average voltage (lots of 1's in data)
    pause(2000)

    strip.set_rgb(1,29,33,0)    'amber
    strip.set_rgb(0,0,0,0)      'turn off greens
    strip.set_rgb(4,29,33,0)    'amber
    strip.set_rgb(5,0,0,0)
    pause(1000)

    strip.set_rgb(2,66,0,0)      'red
    strip.set_rgb(1,0,0,0)       'turn off ambers
    strip.set_rgb(3,66,0,0)      'red
    strip.set_rgb(4,0,0,0)
    pause(2600)
  
      

pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]
 

dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}